from __future__ import annotations

__version__ = "0.1.9"

import asyncio
import pathlib

_OUI_DATA_FILE = pathlib.Path(__file__).parent.joinpath("oui.data")


class OUIManager:
    """Manages the OUI data."""

    def __init__(self) -> None:
        """Initialize the OUIManager."""
        self._oui_to_vendor: dict[str, str] = {}
        self._load_future: asyncio.Future[None] | None = None

    def get_vendor(self, mac: str) -> str | None:
        """Get the vendor for a MAC address."""
        if not self._oui_to_vendor:
            raise RuntimeError("OUI data not loaded, call async_load first")
        return self._oui_to_vendor.get(mac.replace(":", "")[:6].upper())

    async def async_load(self) -> None:
        """Load the OUI data."""
        if self._oui_to_vendor:
            return
        if self._load_future:
            await self._load_future
            return
        loop = asyncio.get_running_loop()
        self._load_future = loop.create_future()
        try:
            self._oui_to_vendor = await loop.run_in_executor(None, self._load_oui_data)
        except Exception as err:
            self._load_future.set_exception(err)
            raise
        else:
            self._load_future.set_result(None)
        finally:
            self._load_future = None

    def _load_oui_data(self) -> dict[str, str]:
        """Load the OUI data."""
        with open(_OUI_DATA_FILE, encoding="utf-8", errors="replace") as f:
            oui_to_vendor: dict[str, str] = {}
            for line in f.read().splitlines():
                oui, _, vendor = line.partition("=")
                oui_to_vendor[oui] = vendor

        return oui_to_vendor


_OUI_MANAGER = OUIManager()


def is_loaded() -> bool:
    """Return if the OUI data is loaded."""
    return bool(_OUI_MANAGER._oui_to_vendor)


def get_vendor(mac: str) -> str | None:
    """Get the vendor for a MAC address."""
    return _OUI_MANAGER.get_vendor(mac)


async def async_load() -> None:
    """Load the OUI data."""
    await _OUI_MANAGER.async_load()


__all__ = ["async_load", "get_vendor", "is_loaded"]



"""Build oui table."""

from __future__ import annotations

import asyncio
import os
import pathlib
import time
from typing import Any

import setuptools


def build(setup_kwargs: dict[str, Any]) -> None:
    """Build the OUI data."""
    setup_kwargs["maintainer"] = "Bluetooth Devices"
    setup_kwargs["maintainer_email"] = "bluetooth@koston.org"
    setup_kwargs["license"] = "MIT"
    setuptools.setup(
        **setup_kwargs,
        long_description_content_type="text/markdown",
        script_args=["bdist_wheel"],
        options={
            "bdist_wheel": {"plat_name": "any"},
        },
    )
    if os.environ.get("AIOOUI_SKIP_REGENERATE"):
        print("Skipping OUI data regeneration.")
        return

    for attempt in range(6):
        try:
            if attempt % 2 == 0:
                asyncio.run(_regenerate_ouis_aiohttp())
            else:
                _regenerate_ouis_requests()
            print(f"Regenerated OUI data on attempt {attempt + 1}.")
            return
        except Exception as e:
            time.sleep(5)
            print(f"Failed to regenerate OUI data: {e}")

    if os.environ.get("AIOOUI_REQUIRE_REGENERATE"):
        raise RuntimeError("Failed to regenerate OUI data")

    print("Using existing data.")


def _regenerate_ouis_requests() -> None:
    import requests  # type: ignore

    resp = requests.get("https://standards-oui.ieee.org/oui.txt", timeout=20)
    resp.raise_for_status()
    _update_from_oui_content(resp.content)


async def _regenerate_ouis_aiohttp() -> None:
    import aiohttp

    async with aiohttp.ClientSession() as session:
        async with session.get("https://standards-oui.ieee.org/oui.txt") as resp:
            resp.raise_for_status()
            _update_from_oui_content(await resp.read())


def _update_from_oui_content(oui_bytes: bytes) -> None:
    oui_to_vendor = {}
    for line in oui_bytes.splitlines():
        if b"(base 16)" in line:
            oui, _, vendor = line.partition(b"(base 16)")
            oui_to_vendor[oui.strip()] = vendor.strip()
    file = pathlib.Path(__file__)
    target_file = file.parent.joinpath("src").joinpath("aiooui").joinpath("oui.data")
    with open(target_file, "wb") as f:
        f.write(b"\n".join(b"=".join((o, v)) for o, v in sorted(oui_to_vendor.items())))


if __name__ == "__main__":
    build({})
